#if defined __APPLE__
#include "SamplesOSX.h"
#else
#include "sample.h"
#endif
#include "sample_common.h"
#include "sprk.h"

#if defined(_MSC_VER)
#include "sprk_exchange.h"
#include "sprk_publish.h"
#endif

#include <iostream>

using namespace HPS;

void Sample::Complex_Clip_Regions(WindowKey wk, Canvas canvas, View view, Model model, SegmentKey modelKey)
{
	/* Sample: Complex_Clip_Regions
	* This sample will show you how to:
	* - Create a grid and use a complex clip region to clip it
	*
	* Read more about the topics covered in this sample in our Programming Guide
	* - section 3.5:		Clip regions
	*/

	/* This block of code sets up the view hierarchy with the necessary attributes to render a grid */
	view.GetSegmentKey().GetCameraControl().SetProjection(HPS::Camera::Projection::Orthographic).SetPosition(HPS::Point(0, 0, 5));
	modelKey.GetVisibilityControl().SetEdges(true).SetVertices(false).SetFaces(false);
	view.GetSegmentKey().GetMaterialMappingControl().SetWindowColor(HPS::RGBColor(1, 1, 1)).SetWindowContrastColor(HPS::RGBColor(1, 1, 1));
	view.GetSegmentKey().GetVisibilityControl().SetFaces(false).SetEdges(true).SetVertices(false).SetMarkers(true);

	/* Here, we are setting up a GridKit. The grid will be inserted twice in overlapping segments. Each grid will have a different color,
	* but only one will be clipped, so you can see the result of the clip operation. */
	HPS::GridKit gridKit;

	gridKit.SetFirstCount(21);
	gridKit.SetSecondCount(21);
	gridKit.SetOrigin(HPS::Point(0, 0, 0));
	gridKit.SetType(HPS::Grid::Type::Quadrilateral);
	gridKit.SetFirstPoint(HPS::Point(0, 1, 0));
	gridKit.SetSecondPoint(HPS::Point(1, 0, 0));

	/* The reference grid is inserted first, in gray. */
	HPS::SegmentKey referenceSegment = modelKey.Subsegment();
	referenceSegment.GetMaterialMappingControl().SetEdgeColor(HPS::RGBColor(0.8f, 0.8f, 0.8f));
	referenceSegment.InsertGrid(gridKit);
	referenceSegment.SetPriority(1);
	
	/* This segment also gets a grid. This grid will be clipped away. */
	HPS::SegmentKey clipSegment = modelKey.Subsegment();
	clipSegment.GetMaterialMappingControl().SetEdgeColor(HPS::RGBColor(1, 0, 0));
	clipSegment.InsertGrid(gridKit);
	clipSegment.SetPriority(2);

	/* Here, we set up the clip regions. */
	PointArray innerLoop;
	innerLoop.push_back(HPS::Point(3, 3, 0));
	innerLoop.push_back(HPS::Point(18, 3, 0));
	innerLoop.push_back(HPS::Point(18, 18, 0));
	innerLoop.push_back(HPS::Point(3, 18, 0));

	PointArray outerLoop;
	outerLoop.push_back(HPS::Point(8, 8, 0));
	outerLoop.push_back(HPS::Point(13, 8, 0));
	outerLoop.push_back(HPS::Point(13, 13, 0));
	outerLoop.push_back(HPS::Point(8, 13, 0));

	PointArrayArray loops;
	loops.push_back(innerLoop);
	loops.push_back(outerLoop);

	clipSegment.GetDrawingAttributeControl().SetClipRegion(loops, Drawing::ClipSpace::World, Drawing::ClipOperation::Remove);

	view.GetSegmentKey().GetMaterialMappingControl().SetEdgeColor(RGBColor(0, 0, 1));

	/* Setting the priority ensures the z-order is correct, since all grids were inserted on the same z-plane. */
	view.GetSegmentKey().SetPriority(3);

	view.FitWorld();
	canvas.Update();


}
