using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using HPS;

namespace hps_cs_samples
{
    public partial class Sample
    {
        public void Hidden_Line(WindowKey wk, Canvas canvas, View view, Model model, SegmentKey modelKey)
        {
            /* Sample: Hidden_Line
	         * This sample will show you how to:
	         * - Load an HSF file through Stream and check for Stream IO exceptions
	         * - Change visibility at the segment level
	         * - Change the rendering algorithm to HiddenLine
	         * - Tweak Hidden Line options
	         * 
	         * Read more about the topics covered in this sample in our Programming Guide 
	         * - section 7.6:	Hidden Surface Removal
	         * - section 7.7:	Hidden Line Removal */

            /* This next block of code shows you how to load an HSF file
             * 1. Create an ImportOptionsKit and set the segment in which you want the file to be loaded
             * 2. Use the Stream::File::Import method to load the desired file, with the options you specified
             * 3. Since this operation happens on a separate thread, wait for it to complete before proceeding 
             * 4. Use a try-catch block to capture any Stream IO exceptions */


            HPS.Stream.ImportNotifier notifier;
	        try
	        {
                HPS.Stream.ImportOptionsKit importOptionsKit = new HPS.Stream.ImportOptionsKit();
		        importOptionsKit.SetSegment(modelKey);
		        string filename = INPUT_FOLDER + "/suspension.hsf";

                notifier = HPS.Stream.File.Import(filename, importOptionsKit);
		        notifier.Wait();
	        }
            catch (HPS.IOException)
	        {
		        //handle Stream IO errors here
	        }

            /* This block shows you how to to set the Hidden Line rendering algorithm
	         * 1. Change the visibility for the segment as needed through a VisibilityControl
	         * 2. Set the rendering algorithm to HiddenLine from the SubwindowControl
	         * 3. Tweak Hidden Line attributes as needed from the HiddenLineAttributeControl object*/
            modelKey.GetVisibilityControl().SetFaces(true).SetEdges(true).SetLines(true);
            modelKey.GetSubwindowControl().SetRenderingAlgorithm(HPS.Subwindow.RenderingAlgorithm.HiddenLine);
            modelKey.GetHiddenLineAttributeControl().SetAlgorithm(HPS.HiddenLine.Algorithm.ZBuffer).SetVisibility(false)
		        .SetSilhouetteCleanup(true).SetRenderFaces(false).SetRenderText(false).SetLinePattern("-  -");

	        view.FitWorld();
	        modelKey.GetModellingMatrixControl().Rotate(0, 90, 0);


        }
    }
}
