#include "hps_test_interface_shared.h"
#include "hps_test_interface.h"

using namespace HPS;

static bool test_options_kit()
{
	CutGeometryGatheringOptionsKit kit1, kit2;
	CuttingSectionKey sh_cutter;
	size_t sh_offset;
	CuttingSection::GatheringLevel sh_level;
	
	if(kit1.ShowCuttingSection(sh_cutter))
		return false;
	if(kit1.ShowOffset(sh_offset))
		return false;
	if(kit1.ShowLevel(sh_level))
		return false;
	
	
	SegmentKey root = Database::CreateRootSegment();
	Plane planes [] = {Plane(0, 0, 1, 0), Plane(1, 0, 0, 1)};
	CuttingSectionKey cutter1 = root.InsertCuttingSection(planes[0]);
	CuttingSectionKey cutter2 = root.InsertCuttingSection(2, planes);

	kit1.SetCuttingSection(cutter1);
	if(!kit1.ShowCuttingSection(sh_cutter)
		|| cutter1 != sh_cutter)
		return false;

	kit1.SetCuttingSection(cutter2);
	if(!kit1.ShowCuttingSection(sh_cutter)
		|| cutter2 != sh_cutter)
		return false;
	
	kit1.UnsetCuttingSection();
	if(kit1.ShowCuttingSection(sh_cutter))
		return false;


	size_t offset1 = 0;
	size_t offset2 = 2;
	kit1.SetOffset(offset1);
	if(!kit1.ShowOffset(sh_offset)
		|| offset1 != sh_offset)
		return false;

	kit1.SetOffset(offset2);
	if(!kit1.ShowOffset(sh_offset)
		|| offset2 != sh_offset)
		return false;

	kit1.UnsetOffset();
	if(kit1.ShowOffset(sh_offset))
		return false;


	CuttingSection::GatheringLevel level1 = CuttingSection::GatheringLevel::Segment;
	CuttingSection::GatheringLevel level2 = CuttingSection::GatheringLevel::SegmentTree;
	kit1.SetLevel(level1);
	if(!kit1.ShowLevel(sh_level)
		|| level1 != sh_level)
		return false;

	kit1.SetLevel(level2);
	if(!kit1.ShowLevel(sh_level)
		|| level2 != sh_level)
		return false;

	kit1.UnsetLevel();
	if(kit1.ShowLevel(sh_level))
		return false;


	kit1.SetCuttingSection(cutter1).SetOffset(offset1).SetLevel(level1);
	kit2 = kit1;
	if(!kit2.ShowCuttingSection(sh_cutter)
		|| sh_cutter != cutter1
		|| !kit2.ShowOffset(sh_offset)
		|| sh_offset != offset1
		|| !kit2.ShowLevel(sh_level)
		|| sh_level != level1)
		return false;
	
	kit1.SetCuttingSection(cutter2).SetOffset(offset2).SetLevel(level2);
	kit2 = kit1;
	if(!kit2.ShowCuttingSection(sh_cutter)
		|| sh_cutter != cutter2
		|| !kit2.ShowOffset(sh_offset)
		|| sh_offset != offset2
		|| !kit2.ShowLevel(sh_level)
		|| sh_level != level2)
		return false;

	kit2.UnsetEverything();
	if(kit2.ShowCuttingSection(sh_cutter))
		return false;
	if(kit2.ShowOffset(sh_offset))
		return false;
	if(kit2.ShowLevel(sh_level))
		return false;


	return true;
}



static bool test_gathering(WindowKey & in_window)
{
	StreamFile file("turbine.hsf");
		
	if(!file.Read(in_window.Subsegment("model")))
		return false;

	CameraKit cam;
	cam.SetPosition(Point(2.67, 1.56f, -0.14f))
		.SetTarget(Point(0, 0, 0))
		.SetUpVector(Vector(-0.4, 0.86, 0.18))
		.SetField(0.86f, 0.86f)
		.SetProjection(Camera::Projection::Orthographic);
	
	in_window.SetCamera(cam);
	in_window.GetMaterialMappingControl().SetWindowColor(RGBAColor(1, 1, 1));

	in_window.GetVisibilityControl()
		.SetVertices(false)
		.SetMarkers(false)
		.SetLines(false)
		.SetEdges(false)
		.SetCutGeometry(true);
	
	in_window.InsertDistantLight(Vector(0.3f, 0.4f, 1));

	in_window.GetPerformanceControl()
		.SetDisplayLists(Performance::DisplayLists::GeometryDisplayLists);

	in_window.GetCuttingSectionAttributeControl()
		.SetCappingLevel(CuttingSection::CappingLevel::Segment)
		.SetCuttingLevel(CuttingSection::CuttingLevel::Global);

	//draw the stuff
	in_window.UpdateWithNotifier(Window::UpdateControl::ExhaustiveUpdate).Wait();

	CuttingSectionKey cutter = in_window.InsertCuttingSection(Plane(Vector(1, 0, 0)));

	//insert plane and look at it again
	in_window.UpdateWithNotifier().Wait();

	CutGeometryGatheringOptionsKit opt;
	opt.SetLevel(CuttingSection::GatheringLevel::SegmentTree);
	
	size_t cap_count = in_window.GatherCutGeometry(in_window.Subsegment("caps"), opt);
	if(cap_count == 0)
		return false;
	
	in_window.Subsegment("model").Delete();
	cutter.Delete();

	//just the caps now
	in_window.UpdateWithNotifier().Wait();

	return true;
}


REGISTER_TEST(test_00000_gather_cut_geometry);

RESULT test_00000_gather_cut_geometry(driver_data * dtt)
{
	if(!test_options_kit())
		return Test::Failure;

	if(!test_gathering(dtt->window))
		return Test::Failure;

	return Test::Capture;
}



