var allLinks = {};

function getAncestorWithTagName($element, tagName) {
	for (var $parent = $element.parent(); $parent.length > 0; $parent = $parent.parent()) {
		var parentTagName = $parent.prop('tagName');
		if (parentTagName) {
			if (parentTagName.toLowerCase() == tagName.toLowerCase())
				return $parent;
		}
	}
	
	return null;
}

function selectAccordionSectionContainingElement($element) {
	$section = getAncestorWithTagName($element, 'div');
	$accordion = $section ? $section.parent() : null;
	if ($section && $accordion && $accordion.length > 0) {
		var sectionIndex = Math.floor($section.index() / 2);	
		$accordion.accordion({active : sectionIndex});			
		selectAccordionSectionContainingElement($accordion);
	}
}

function _handleTocLinkClick(path) {
	var handledLink = null;	
	var touchedLinks = {};
	
	$('toc-container div, .toc-container div').each(function(sectionIndex, sectionItem) {
		$(this).find('a').each(function() {
			if (!(this in touchedLinks)) {
				touchedLinks[this] = true;
				
				var $link = $(this);	
				var href = $link.attr('href');			
				if (href == path) {		
					handledLink = path;
					
					selectAccordionSectionContainingElement($link);
					
					if (!$link.hasClass('selected-link'))
						$link.addClass('selected-link');
				} else {
					$link.removeClass('selected-link');
				}			
			}
		});
	});
	
	return handledLink;
}

function _handleInternalLinkClick(path) {
	var result = null;
	
	if (!result) {	
		var membersMatch = new RegExp('/class.*-members\\.html').exec(path);
		if (membersMatch)
			result = _handleTocLinkClick(path.replace(membersMatch, '/functions.html'));		
	}
	
	if (!result) {
		var classMatch = new RegExp('/class.*\\.html').exec(path);
		if (classMatch)
			result = _handleTocLinkClick(path.replace(classMatch, '/classes.html'));		
	}
	
	if (!result) {
		var guideCode = new RegExp('/guides/code/.*').exec(path);
		if (guideCode)
			result = _handleTocLinkClick(path.replace(guideCode, '/guides/sample_code.html'));
	}
	
	return result;
}

function handleLinkClick(path) {
	var result = null;
	
	if (path.indexOf('file:') == 0) {
		var parts = path.split('/');
		for (var i = 0; i < parts.length; i++) {
			if (parts[i].length > 0) {
				result = _handleTocLinkClick(parts[i]);
				if (result)
					break;
			}			
			
			result = _handleInternalLinkClick(parts[i]);
			if (result)
				break;
		}
	} else if (path in allLinks) {
		result = _handleTocLinkClick(path);
	} else {
		result = _handleInternalLinkClick(path);
	}
	
	return result;
}

function handleMessage(e) {
	var path = e.data.path.split('#')[0];
	handleLinkClick(path);
}

if (window.attachEvent)
	window.attachEvent('message', handleMessage);
else
	window.addEventListener('message', handleMessage, false);

$(document).ready(function() {	
	$('toc-container, .toc-container').accordion({
		collapsible: true,
		heightStyle: 'content',
		active: false
	});
	

		
	$('toc-container a, .toc-container a').each(function() {
		var href = $(this).attr('href');
		if (href)
			allLinks[href] = true;
	});
	
	$('toc-container a, .toc-container a').click(function(e) {
		var href = $(this).attr('href');
		if (href && href.indexOf('http') != 0) {
			var title = $(this).text();
			
			window.top.postMessage({'path' : href, 'title' : title}, '*');
			
			handleLinkClick(href);
			
			e.preventDefault();
		}
	});
});